@@ -6,8 +6,7 @@
 /**
  * Value object holding dqlite configuration.
  */
-struct config
-{
+struct config {
 	dqlite_node_id id;             /* Unique instance ID */
 	char *address;                 /* Instance address */
 	unsigned heartbeat_timeout;    /* In milliseconds */
@@ -17,10 +16,12 @@ struct config
 	char name[256];                /* VFS/replication registriatio name */
 	unsigned long long failure_domain; /* User-provided failure domain */
 	unsigned long long int weight;     /* User-provided node weight */
-	char dir[1024];                    /* Data dir for on-disk database */
+	char raft_dir[1024];               /* Directory used by raft */
+	char database_dir[1024];           /* Data dir for on-disk database */
 	bool disk;                         /* Disk-mode or not */
 	int voters;                        /* Target number of voters */
 	int standbys;                      /* Target number of standbys */
+	unsigned pool_thread_count;    /* Number of threads in thread pool */
 };
 
 /**
@@ -30,7 +31,8 @@ struct config
 int config__init(struct config *c,
 		 dqlite_node_id id,
 		 const char *address,
-		 const char *dir);
+		 const char *raft_dir,
+		 const char *database_dir);
 
 /**
  * Release any memory held by the config object.
