@@ -385,13 +385,15 @@ static int encodeDatabase(struct db *db,
 {
 	struct snapshotDatabase header;
 	sqlite3_vfs *vfs;
-	uint32_t database_size = 0;
-	uint8_t *page;
 	char *cursor;
 	struct dqlite_buffer *bufs = (struct dqlite_buffer *)r_bufs;
 	int rv;
 
 	header.filename = db->filename;
+	header.main_size = (n - 1) * (uint64_t)db->config->page_size;
+	/* The database is checkpointed before writing it to disk.  As such,
+	 * wal_size is always 0. */
+	header.wal_size = 0;
 
 	vfs = sqlite3_vfs_find(db->config->name);
 	rv = VfsShallowSnapshot(vfs, db->filename, &bufs[1], n - 1);
@@ -399,32 +401,18 @@ static int encodeDatabase(struct db *db,
 		goto err;
 	}
 
-	/* Extract the database size from the first page. */
-	page = bufs[1].base;
-	database_size += (uint32_t)(page[28] << 24);
-	database_size += (uint32_t)(page[29] << 16);
-	database_size += (uint32_t)(page[30] << 8);
-	database_size += (uint32_t)(page[31]);
-
-	header.main_size =
-	    (uint64_t)database_size * (uint64_t)db->config->page_size;
-	header.wal_size = bufs[n - 1].len;
-
 	/* Database header. */
 	bufs[0].len = snapshotDatabase__sizeof(&header);
 	bufs[0].base = sqlite3_malloc64(bufs[0].len);
 	if (bufs[0].base == NULL) {
 		rv = RAFT_NOMEM;
-		goto err_after_snapshot;
+		goto err;
 	}
 	cursor = bufs[0].base;
 	snapshotDatabase__encode(&header, &cursor);
 
 	return 0;
 
-err_after_snapshot:
-	/* Free the wal buffer */
-	sqlite3_free(bufs[n - 1].base);
 err:
 	assert(rv != 0);
 	return rv;
@@ -490,7 +478,7 @@ static unsigned dbNumPages(struct db *db)
 	uint32_t n;
 
 	vfs = sqlite3_vfs_find(db->config->name);
-	rv = VfsDatabaseNumPages(vfs, db->filename, &n);
+	rv = VfsDatabaseNumPages(vfs, db->filename, true, &n);
 	assert(rv == 0);
 	return n;
 }
@@ -502,10 +490,10 @@ static unsigned snapshotNumBufs(struct fsm *f)
 	queue *head;
 	unsigned n = 1; /* snapshot header */
 
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		n += 2; /* database header & wal */
-		db = QUEUE__DATA(head, struct db, queue);
+		n += 1; /* database header */
+		db = QUEUE_DATA(head, struct db, queue);
 		n += dbNumPages(db); /* 1 buffer per page (zero copy) */
 	}
 
@@ -514,13 +502,12 @@ static unsigned snapshotNumBufs(struct fsm *f)
 
 /* An example array of snapshot buffers looks like this:
  *
- * bufs:  SH DH1 P1 P2 P3 WAL1 DH2 P1 P2 WAL2
- * index:  0   1  2  3  4    5   6  7  8    9
+ * bufs:  SH DH1 P1 P2 P3 DH2 P1 P2
+ * index:  0   1  2  3  4   6  7  8
  *
  * SH:   Snapshot Header
  * DHx:  Database Header
  * Px:   Database Page (not to be freed)
- * WALx: a WAL
  * */
 static void freeSnapshotBufs(struct fsm *f,
 			     struct raft_buffer bufs[],
@@ -539,18 +526,16 @@ static void freeSnapshotBufs(struct fsm *f,
 
 	i = 1;
 	/* Free all database headers & WAL buffers */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		if (i == n_bufs) {
 			break;
 		}
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		/* i is the index of the database header */
 		sqlite3_free(bufs[i].base);
 		/* i is now the index of the next database header (if any) */
-		i += 1 /* db header */ + dbNumPages(db) + 1 /* WAL */;
-		/* free WAL buffer */
-		sqlite3_free(bufs[i - 1].base);
+		i += 1 /* db header */ + dbNumPages(db);
 	}
 }
 
@@ -567,9 +552,9 @@ static int fsm__snapshot(struct raft_fsm *fsm,
 
 	/* First count how many databases we have and check that no transaction
 	 * nor checkpoint nor other snapshot is in progress. */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		if (db->tx_id != 0 || db->read_lock) {
 			return RAFT_BUSY;
 		}
@@ -577,9 +562,9 @@ static int fsm__snapshot(struct raft_fsm *fsm,
 	}
 
 	/* Lock all databases, preventing the checkpoint from running */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		rv = databaseReadLock(db);
 		assert(rv == 0);
 	}
@@ -598,11 +583,11 @@ static int fsm__snapshot(struct raft_fsm *fsm,
 
 	/* Encode individual databases. */
 	i = 1;
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
-		/* database_header + num_pages + wal */
-		unsigned n = 1 + dbNumPages(db) + 1;
+		db = QUEUE_DATA(head, struct db, queue);
+		/* database_header + num_pages */
+		unsigned n = 1 + dbNumPages(db);
 		rv = encodeDatabase(db, &(*bufs)[i], n);
 		if (rv != 0) {
 			goto err_after_encode_header;
@@ -618,9 +603,9 @@ err_after_encode_header:
 err_after_bufs_alloc:
 	sqlite3_free(*bufs);
 err:
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		databaseReadUnlock(db);
 	}
 	assert(rv != 0);
@@ -663,12 +648,12 @@ static int fsm__snapshot_finalize(struct raft_fsm *fsm,
 	/* Unlock all databases that were locked for the snapshot, this is safe
 	 * because DB's are only ever added at the back of the queue. */
 	n_db = 0;
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		if (n_db == header.n) {
 			break;
 		}
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		rv = databaseReadUnlock(db);
 		assert(rv == 0);
 		n_db++;
@@ -815,7 +800,7 @@ static unsigned snapshotNumBufsDisk(struct fsm *f)
 	queue *head;
 	unsigned n = 1; /* snapshot header */
 
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		n += 3; /* database header, database file and wal */
 	}
@@ -849,7 +834,7 @@ static void freeSnapshotBufsDisk(struct fsm *f,
 
 	i = 1;
 	/* Free all database headers & WAL buffers. Unmap the DB file. */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		if (i == n_bufs) {
 			break;
@@ -878,9 +863,9 @@ static int fsm__snapshot_disk(struct raft_fsm *fsm,
 
 	/* First count how many databases we have and check that no transaction
 	 * nor checkpoint nor other snapshot is in progress. */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		if (db->tx_id != 0 || db->read_lock) {
 			return RAFT_BUSY;
 		}
@@ -890,9 +875,9 @@ static int fsm__snapshot_disk(struct raft_fsm *fsm,
 	/* Lock all databases, preventing the checkpoint from running. This
 	 * ensures the database is not written while it is mmap'ed and copied by
 	 * raft. */
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		rv = databaseReadLock(db);
 		assert(rv == 0);
 	}
@@ -917,9 +902,9 @@ static int fsm__snapshot_disk(struct raft_fsm *fsm,
 
 	/* Copy WAL of all databases. */
 	i = 1;
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		/* database_header + db + WAL */
 		unsigned n = 3;
 		/* pass pointer to buffer that will contain WAL. */
@@ -938,9 +923,9 @@ err_after_encode_sync:
 err_after_bufs_alloc:
 	sqlite3_free(*bufs);
 err:
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		databaseReadUnlock(db);
 	}
 	assert(rv != 0);
@@ -972,13 +957,13 @@ static int fsm__snapshot_async_disk(struct raft_fsm *fsm,
 
 	/* Encode individual databases. */
 	i = 1;
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		if (i == *n_bufs) {
 			/* In case a db was added in meanwhile */
 			break;
 		}
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		/* database_header + database file + wal */
 		unsigned n = 3;
 		rv = encodeDiskDatabaseAsync(db, &(*bufs)[i], n);
@@ -1031,12 +1016,12 @@ static int fsm__snapshot_finalize_disk(struct raft_fsm *fsm,
 	/* Unlock all databases that were locked for the snapshot, this is safe
 	 * because DB's are only ever added at the back of the queue. */
 	n_db = 0;
-	QUEUE__FOREACH(head, &f->registry->dbs)
+	QUEUE_FOREACH(head, &f->registry->dbs)
 	{
 		if (n_db == header.n) {
 			break;
 		}
-		db = QUEUE__DATA(head, struct db, queue);
+		db = QUEUE_DATA(head, struct db, queue);
 		databaseReadUnlock(db);
 		n_db++;
 	}
