@@ -12,7 +12,6 @@
 #include "client/protocol.h"
 #include "conn.h"
 #include "fsm.h"
-#include "id.h"
 #include "lib/addr.h"
 #include "lib/assert.h"
 #include "lib/fs.h"
@@ -60,8 +59,6 @@ int dqlite__init(struct dqlite_node *d,
 {
 	int rv;
 	char db_dir_path[1024];
-	int urandom;
-	ssize_t count;
 
 	d->initialized = false;
 	d->lock_fd = -1;
@@ -94,16 +91,6 @@ int dqlite__init(struct dqlite_node *d,
 		rv = DQLITE_ERROR;
 		goto err_after_vfs_init;
 	}
-#ifdef DQLITE_NEXT
-	rv = pool_init(&d->pool, &d->loop, d->config.pool_thread_count,
-		       POOL_QOS_PRIO_FAIR);
-	if (rv != 0) {
-		snprintf(d->errmsg, DQLITE_ERRMSG_BUF_SIZE, "pool_init(): %s",
-			 uv_strerror(rv));
-		rv = DQLITE_ERROR;
-		goto err_after_loop_init;
-	}
-#endif
 	rv = raftProxyInit(&d->raft_transport, &d->loop);
 	if (rv != 0) {
 		goto err_after_pool_init;
@@ -171,11 +158,6 @@ int dqlite__init(struct dqlite_node *d,
 	d->connect_func = transportDefaultConnect;
 	d->connect_func_arg = NULL;
 
-	urandom = open("/dev/urandom", O_RDONLY);
-	assert(urandom != -1);
-	count = read(urandom, d->random_state.data, sizeof(uint64_t[4]));
-	(void)count;
-	close(urandom);
 	d->initialized = true;
 	return 0;
 
@@ -190,11 +172,6 @@ err_after_raft_io_init:
 err_after_raft_transport_init:
 	raftProxyClose(&d->raft_transport);
 err_after_pool_init:
-#ifdef DQLITE_NEXT
-	pool_close(&d->pool);
-	pool_fini(&d->pool);
-err_after_loop_init:
-#endif
 	uv_loop_close(&d->loop);
 err_after_vfs_init:
 	VfsClose(&d->vfs);
@@ -222,9 +199,6 @@ void dqlite__close(struct dqlite_node *d)
 	// the TODO above referencing the cleanup logic without running the
 	// node. See https://github.com/canonical/dqlite/issues/504.
 
-#ifdef DQLITE_NEXT
-	pool_fini(&d->pool);
-#endif
 	uv_loop_close(&d->loop);
 	raftProxyClose(&d->raft_transport);
 	registry__close(&d->registry);
@@ -559,9 +533,6 @@ static void stopCb(uv_async_t *stop)
 		tracef("not running or already stopped");
 		return;
 	}
-#ifdef DQLITE_NEXT
-	pool_close(&d->pool);
-#endif
 	if (d->role_management) {
 		rv = uv_timer_stop(&d->timer);
 		assert(rv == 0);
@@ -595,7 +566,6 @@ static void listenCb(uv_stream_t *listener, int status)
 	struct dqlite_node *t = listener->data;
 	struct uv_stream_s *stream;
 	struct conn *conn;
-	struct id_state seed;
 	int rv;
 
 	if (!t->running) {
@@ -665,15 +635,12 @@ static void listenCb(uv_stream_t *listener, int status)
 #endif
 	}
 
-	seed = t->random_state;
-	idJump(&t->random_state);
-
 	conn = sqlite3_malloc(sizeof *conn);
 	if (conn == NULL) {
 		goto err;
 	}
 	rv = conn__start(conn, &t->config, &t->loop, &t->registry, &t->raft,
-			 stream, &t->raft_transport, seed, destroy_conn);
+			 stream, &t->raft_transport, destroy_conn);
 	if (rv != 0) {
 		goto err_after_conn_alloc;
 	}
@@ -1022,6 +989,7 @@ int dqlite_node_recover_ext(dqlite_node *n,
 			    struct dqlite_node_info_ext infos[],
 			    int n_info)
 {
+	dqliteTracingMaybeEnable(true);
 	tracef("dqlite node recover ext");
 	struct raft_configuration configuration;
 	int i;
@@ -1031,6 +999,7 @@ int dqlite_node_recover_ext(dqlite_node *n,
 	for (i = 0; i < n_info; i++) {
 		struct dqlite_node_info_ext *info = &infos[i];
 		if (!node_info_valid(info)) {
+			tracef("invalid node info");
 			rv = DQLITE_MISUSE;
 			goto out;
 		}
@@ -1040,6 +1009,7 @@ int dqlite_node_recover_ext(dqlite_node *n,
 		rv = raft_configuration_add(&configuration, info->id, address,
 					    raft_role);
 		if (rv != 0) {
+			tracef("unable to add server to raft configuration, error: %d", rv);
 			assert(rv == RAFT_NOMEM);
 			rv = DQLITE_NOMEM;
 			goto out;
@@ -1049,11 +1019,15 @@ int dqlite_node_recover_ext(dqlite_node *n,
 	int lock_fd;
 	rv = acquire_dir(n->config.raft_dir, &lock_fd);
 	if (rv != 0) {
+		tracef("couldn't acquire lock, error: %d", rv);
 		goto out;
 	}
 
 	rv = raft_recover(&n->raft, &configuration);
 	if (rv != 0) {
+		tracef("raft recovery failed, error: %d", rv);
+		snprintf(n->errmsg, DQLITE_ERRMSG_BUF_SIZE, "raft_recover(): %s",
+			 raft_errmsg(&n->raft));
 		rv = DQLITE_ERROR;
 		goto out;
 	}
@@ -1065,6 +1039,23 @@ out:
 	return rv;
 }
 
+int dqlite_node_describe_last_entry(dqlite_node *n,
+				    uint64_t *index,
+				    uint64_t *term)
+{
+	PRE(n->initialized && !n->running);
+	static_assert(sizeof(*index) == sizeof(raft_index),
+		      "unexpected index type size");
+	raft_index *i = (raft_index *)index;
+	static_assert(sizeof(*term) == sizeof(raft_term),
+		      "unexpected term type size");
+	raft_term *t = (raft_term *)term;
+	int rv;
+
+	rv = raft_io_describe_last_entry(&n->raft_io, i, t);
+	return rv == 0 ? 0 : DQLITE_ERROR;
+}
+
 dqlite_node_id dqlite_generate_node_id(const char *address)
 {
 	tracef("generate node id");
