@@ -1228,20 +1228,20 @@ static int vfsFileCheckReservedLock(sqlite3_file *file, int *result)
 
 /* Handle pragma a pragma file control. See the xFileControl
  * docstring in sqlite.h.in for more details. */
-static int vfsFileControlPragma(struct vfsFile *f, char **fnctl)
+static int vfsFileControlPragma(struct vfsFile *f, char **fcntl)
 {
 	const char *left;
 	const char *right;
 
 	assert(f != NULL);
-	assert(fnctl != NULL);
+	assert(fcntl != NULL);
 
-	left = fnctl[1];
-	right = fnctl[2];
+	left = fcntl[1];
+	right = fcntl[2];
 
 	assert(left != NULL);
 
-	if (strcmp(left, "page_size") == 0 && right) {
+	if (sqlite3_stricmp(left, "page_size") == 0 && right) {
 		/* When the user executes 'PRAGMA page_size=N' we save the
 		 * size internally.
 		 *
@@ -1261,19 +1261,23 @@ static int vfsFileControlPragma(struct vfsFile *f, char **fnctl)
 			if (f->database->n_pages > 0 &&
 			    page_size !=
 				(int)vfsDatabaseGetPageSize(f->database)) {
-				fnctl[0] = sqlite3_mprintf(
+				fcntl[0] = sqlite3_mprintf(
 				    "changing page size is not supported");
 				return SQLITE_IOERR;
 			}
 		}
-	} else if (strcmp(left, "journal_mode") == 0 && right) {
+	} else if (sqlite3_stricmp(left, "journal_mode") == 0 && right) {
 		/* When the user executes 'PRAGMA journal_mode=x' we ensure
 		 * that the desired mode is 'wal'. */
 		if (strcasecmp(right, "wal") != 0) {
-			fnctl[0] =
+			fcntl[0] =
 			    sqlite3_mprintf("only WAL mode is supported");
 			return SQLITE_IOERR;
 		}
+	} else if (sqlite3_stricmp(left, "wal_checkpoint") == 0 
+			|| (sqlite3_stricmp(left, "wal_autocheckpoint") == 0 && right)) {
+		fcntl[0] = sqlite3_mprintf("custom checkpoint not allowed");
+		return SQLITE_IOERR;
 	}
 
 	/* We're returning NOTFOUND here to tell SQLite that we wish it to go on
@@ -2449,7 +2453,23 @@ static uint32_t vfsDatabaseGetNumberOfPages(struct vfsDatabase *d)
 	return ByteGetBe32(&page[28]);
 }
 
-int VfsDatabaseNumPages(sqlite3_vfs *vfs, const char *filename, uint32_t *n)
+static uint32_t vfsDatabaseNumPages(struct vfsDatabase *database, bool use_wal) {
+	uint32_t n;
+	if (use_wal && database->wal.n_frames > 0) {
+		n = vfsFrameGetDatabaseSize(database->wal.frames[database->wal.n_frames-1]);
+		/* If the result is zero, it means that the WAL contains
+		 * uncommitted transactions. */
+		POST(n > 0);
+	} else {
+		n = vfsDatabaseGetNumberOfPages(database);
+	}
+	return n;
+}
+
+int VfsDatabaseNumPages(sqlite3_vfs *vfs,
+			const char *filename,
+			bool use_wal,
+			uint32_t *n)
 {
 	struct vfs *v;
 	struct vfsDatabase *d;
@@ -2460,10 +2480,11 @@ int VfsDatabaseNumPages(sqlite3_vfs *vfs, const char *filename, uint32_t *n)
 		return -1;
 	}
 
-	*n = vfsDatabaseGetNumberOfPages(d);
+	*n = vfsDatabaseNumPages(d, use_wal);
 	return 0;
 }
 
+
 static void vfsDatabaseSnapshot(struct vfsDatabase *d, uint8_t **cursor)
 {
 	uint32_t page_size;
@@ -2545,20 +2566,46 @@ int VfsSnapshot(sqlite3_vfs *vfs, const char *filename, void **data, size_t *n)
 }
 
 static void vfsDatabaseShallowSnapshot(struct vfsDatabase *d,
-				       struct dqlite_buffer *bufs)
+				       struct dqlite_buffer *bufs, uint32_t n)
 {
 	uint32_t page_size;
 
 	page_size = vfsDatabaseGetPageSize(d);
 	assert(page_size > 0);
 
+	if (d->n_pages < n) {
+		n = d->n_pages;
+	}
+
 	/* Fill the buffers with pointers to all of the database pages */
-	for (unsigned i = 0; i < d->n_pages; ++i) {
+	for (unsigned i = 0; i < n; ++i) {
 		bufs[i].base = d->pages[i];
 		bufs[i].len = page_size;
 	}
 }
 
+static void vfsWalShallowSnapshot(struct vfsWal *w,
+				  struct dqlite_buffer *bufs,
+				  uint32_t n) {
+	uint32_t page_size;
+	unsigned i;
+
+	if (w->n_frames == 0) {
+		return;
+	}
+
+	page_size = vfsWalGetPageSize(w);
+	assert(page_size > 0);
+
+	for (i = 0; i < w->n_frames; i++) {
+		struct vfsFrame *frame = w->frames[i];
+		uint32_t page_number = vfsFrameGetPageNumber(frame);
+		assert(page_number <= n);
+		bufs[page_number - 1].base = frame->page;
+		bufs[page_number - 1].len = page_size;
+	}
+}
+
 int VfsShallowSnapshot(sqlite3_vfs *vfs,
 		       const char *filename,
 		       struct dqlite_buffer bufs[],
@@ -2567,8 +2614,6 @@ int VfsShallowSnapshot(sqlite3_vfs *vfs,
 	tracef("vfs snapshot filename %s", filename);
 	struct vfs *v;
 	struct vfsDatabase *database;
-	struct vfsWal *wal;
-	uint8_t *cursor;
 
 	v = (struct vfs *)(vfs->pAppData);
 	database = vfsDatabaseLookup(v, filename);
@@ -2583,24 +2628,15 @@ int VfsShallowSnapshot(sqlite3_vfs *vfs,
 		return SQLITE_CORRUPT;
 	}
 
-	if (database->n_pages != n - 1) {
+	if (vfsDatabaseNumPages(database, true) != n) {
 		tracef("not enough buffers provided");
 		return SQLITE_MISUSE;
 	}
 
-	/* Copy WAL to last buffer. */
-	wal = &database->wal;
-	bufs[n - 1].len = vfsWalFileSize(wal);
-	bufs[n - 1].base = sqlite3_malloc64(bufs[n - 1].len);
-	/* WAL can have 0 length! */
-	if (bufs[n - 1].base == NULL && bufs[n - 1].len != 0) {
-		return SQLITE_NOMEM;
-	}
-	cursor = bufs[n - 1].base;
-	vfsWalSnapshot(wal, &cursor);
+	vfsDatabaseShallowSnapshot(database, bufs, n);
+	/* Update the bufs array with newer versions of pages from the WAL. */
+	vfsWalShallowSnapshot(&database->wal, bufs, n);
 
-	/* Copy page pointers to first n-1 buffers */
-	vfsDatabaseShallowSnapshot(database, bufs);
 	return 0;
 }
 
@@ -3020,17 +3056,17 @@ static int vfsDiskFileCheckReservedLock(sqlite3_file *file, int *result)
 
 /* Handle pragma a pragma file control. See the xFileControl
  * docstring in sqlite.h.in for more details. */
-static int vfsDiskFileControlPragma(struct vfsFile *f, char **fnctl)
+static int vfsDiskFileControlPragma(struct vfsFile *f, char **fcntl)
 {
 	int rv;
 	const char *left;
 	const char *right;
 
 	assert(f != NULL);
-	assert(fnctl != NULL);
+	assert(fcntl != NULL);
 
-	left = fnctl[1];
-	right = fnctl[2];
+	left = fcntl[1];
+	right = fcntl[2];
 
 	assert(left != NULL);
 
@@ -3041,22 +3077,22 @@ static int vfsDiskFileControlPragma(struct vfsFile *f, char **fnctl)
 		 * Only used for on-disk databases.
 		 * */
 		if (f->db == NULL) {
-			fnctl[0] = sqlite3_mprintf("no DB file found");
+			fcntl[0] = sqlite3_mprintf("no DB file found");
 			return SQLITE_IOERR;
 		}
 		if (page_size > UINT16_MAX) {
-			fnctl[0] = sqlite3_mprintf("max page_size exceeded");
+			fcntl[0] = sqlite3_mprintf("max page_size exceeded");
 			return SQLITE_IOERR;
 		}
 		if (f->database->page_size == 0) {
 			rv = f->db->pMethods->xFileControl(
-			    f->db, SQLITE_FCNTL_PRAGMA, fnctl);
+			    f->db, SQLITE_FCNTL_PRAGMA, fcntl);
 			if (rv == SQLITE_NOTFOUND || rv == SQLITE_OK) {
 				f->database->page_size = (uint16_t)page_size;
 			}
 			return rv;
 		} else if ((uint16_t)page_size != f->database->page_size) {
-			fnctl[0] = sqlite3_mprintf(
+			fcntl[0] = sqlite3_mprintf(
 			    "changing page size is not supported");
 			return SQLITE_IOERR;
 		}
@@ -3064,7 +3100,7 @@ static int vfsDiskFileControlPragma(struct vfsFile *f, char **fnctl)
 		/* When the user executes 'PRAGMA journal_mode=x' we ensure
 		 * that the desired mode is 'wal'. */
 		if (strcasecmp(right, "wal") != 0) {
-			fnctl[0] =
+			fcntl[0] =
 			    sqlite3_mprintf("only WAL mode is supported");
 			return SQLITE_IOERR;
 		}
@@ -3482,6 +3518,25 @@ err:
 	return rv;
 }
 
+int VfsSnapshotDisk(sqlite3_vfs *vfs,
+		    const char *filename,
+		    struct dqlite_buffer bufs[],
+		    unsigned n)
+{
+	int rv;
+	if (n != 2) {
+		return -1;
+	}
+
+	rv = VfsDiskSnapshotDb(vfs, filename, &bufs[0]);
+	if (rv != 0) {
+		return rv;
+	}
+
+	rv = VfsDiskSnapshotWal(vfs, filename, &bufs[1]);
+	return rv;
+}
+
 static int vfsDiskDatabaseRestore(struct vfsDatabase *d,
 				  const char *filename,
 				  const uint8_t *data,
